@@ -1,4 +1,4 @@
-/*****
+                        /*****
 *
 * Copyright (C) 2001, 2002, 2003, 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
@@ -35,8 +35,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
+#include <signal.h>
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
 # include <pwd.h>
 # include <grp.h>
 #endif
@@ -66,16 +67,13 @@
 #include "tls-register.h"
 
 
-#ifdef WIN32
+#if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
 # define chown(x, y, z) (0)
 # define fchown(x, y, z) (0)
 # define getuid(x) (0)
 # define getgid(x) (0)
-# define mkdir(x, y) mkdir(x)
 #endif
 
-#define TLS_CONFIG PRELUDE_CONFIG_DIR "/default/tls.conf"
-
 
 struct rm_dir_s {
         prelude_list_t list;
@@ -108,6 +106,22 @@ int generated_certificate_lifetime = 0;
 static int64_t offset = -1, count = -1;
 
 
+static const char *get_tls_config(void)
+{
+        static char buf[PATH_MAX];
+        static int initialized = 0;
+
+        if ( initialized )
+                return buf;
+
+        prelude_client_profile_get_default_config_dirname(NULL, buf, sizeof(buf));
+        snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "/tls.conf");
+        initialized = 1;
+
+        return buf;
+}
+
+
 static int chown_cb(const char *filename, const struct stat *st, int flag)
 {
         int ret;
@@ -151,6 +165,7 @@ static int do_chown(const char *name)
 
 static void permission_warning(void)
 {
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         fprintf(stderr,
 "* WARNING: no --uid or --gid command line options were provided.\n*\n"
 
@@ -163,6 +178,7 @@ static void permission_warning(void)
                 (int) prelude_client_profile_get_uid(profile), (int) prelude_client_profile_get_gid(profile));
 
         while ( getchar() != '\n' );
+#endif
 }
 
 
@@ -205,12 +221,12 @@ static void print_tls_settings(void)
 
 fprintf(stderr,
 
-"TLS Options (from: " TLS_CONFIG "):\n"
+"TLS Options (from: %s):\n"
 "  --key-len=LEN           : Profile private key length (default: %d bits).\n"
 "  --cert-lifetime=DAYS    : Profile certificate lifetime (default: %s).\n"
 "  --ca-cert-lifetime=DAYS : Authority certificate lifetime (default: %s).\n"
 
-"\n", generated_key_size, buf1, buf2);
+"\n", get_tls_config(), generated_key_size, buf1, buf2);
 }
 
 
@@ -245,7 +261,7 @@ static void print_registration_server_help(void)
 "to communicate with Prelude-Manager instance using this profile.\n\n"
 
 "Options:\n"
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
 "  --uid=UID            : UID or user used to create the analyzer profile.\n"
 "  --gid=GID            : GID or group used to create the analyzer profile.\n"
 #endif
@@ -271,7 +287,7 @@ static void print_register_help(void)
 "registration server. The analyzer profile will be created if it does not exist.\n\n"
 
 "Options:\n"
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
 "  --uid=UID             : UID or user used to create analyzer profile.\n"
 "  --gid=GID             : GID or group used to create analyzer profile.\n"
 #endif
@@ -291,7 +307,7 @@ static void print_add_help(void)
 
 "This command will create the specified analyzer profile.\n\n"
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
 "Options:\n"
 "  --uid=UID: UID or user used to create analyzer profile.\n"
 "  --gid=GID: GID or group used to create analyzer profile.\n"
@@ -310,7 +326,7 @@ static void print_chown_help(void)
 "Usage  : chown <analyzer profile> <--uid|--gid>\n"
 "Example: chown prelude-lml --uid lmluser --gid lmlgroup\n\n"
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
 "Options:\n"
 "  --uid=UID : UID or user used as new profile permission.\n"
 "  --gid=GID : GID to group used as new profile permission.\n"
@@ -384,7 +400,7 @@ static void print_send_help(void)
 }
 
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
 static int set_uid(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
 {
         uid_t uid;
@@ -565,7 +581,7 @@ static int set_long_listing(prelude_option_t *opt, const char *optarg, prelude_s
 
 static void setup_permission_options(void)
 {
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         prelude_option_add(parentopt, NULL, PRELUDE_OPTION_TYPE_CLI, 'u', "uid",
                            NULL, PRELUDE_OPTION_ARGUMENT_REQUIRED, set_uid, NULL);
 
@@ -599,16 +615,16 @@ static int read_tls_setting(void)
         config_t *cfg;
         unsigned int line;
 
-        ret = _config_open(&cfg, TLS_CONFIG);
+        ret = _config_open(&cfg, get_tls_config());
         if ( ret < 0 ) {
-                prelude_perror(ret, "could not open %s", TLS_CONFIG);
+                prelude_perror(ret, "could not open %s", get_tls_config());
                 return -1;
         }
 
         line = 0;
         ptr = _config_get(cfg, NULL, "generated-key-size", &line);
         if ( ! ptr ) {
-                fprintf(stderr, "%s: couldn't find \"generated-key-size\" setting.\n", TLS_CONFIG);
+                fprintf(stderr, "%s: couldn't find \"generated-key-size\" setting.\n", get_tls_config());
                 goto err;
         }
 
@@ -618,7 +634,7 @@ static int read_tls_setting(void)
         line = 0;
         ptr = _config_get(cfg, NULL, "authority-certificate-lifetime", &line);
         if ( ! ptr ) {
-                fprintf(stderr, "%s: couldn't find \"authority-certificate-lifetime\" setting.\n", TLS_CONFIG);
+                fprintf(stderr, "%s: couldn't find \"authority-certificate-lifetime\" setting.\n", get_tls_config());
                 goto err;
         }
 
@@ -628,7 +644,7 @@ static int read_tls_setting(void)
         line = 0;
         ptr = _config_get(cfg, NULL, "generated-certificate-lifetime", &line);
         if ( ! ptr ) {
-                fprintf(stderr, "%s: couldn't find \"generated-certificate-lifetime\" setting.\n", TLS_CONFIG);
+                fprintf(stderr, "%s: couldn't find \"generated-certificate-lifetime\" setting.\n", get_tls_config());
                 goto err;
         }
 
@@ -645,7 +661,11 @@ static int read_tls_setting(void)
 
 static void setup_tls_options(void)
 {
-        read_tls_setting();
+        int ret;
+
+        ret = read_tls_setting();
+        if ( ret < 0 )
+                exit(1);
 
         prelude_option_add(parentopt, NULL, PRELUDE_OPTION_TYPE_CLI, 0, "key-len",
                            NULL, PRELUDE_OPTION_ARGUMENT_REQUIRED, set_key_len, NULL);
@@ -688,7 +708,10 @@ static int create_template_config_file(prelude_client_profile_t *profile)
         int ret;
         FILE *fd, *tfd;
         size_t len, wlen;
-        char filename[PATH_MAX], buf[8192];
+        char dirname[PATH_MAX], filename[PATH_MAX], buf[8192];
+
+        prelude_client_profile_get_default_config_dirname(profile, dirname, sizeof(dirname));
+        snprintf(dirname + strlen(dirname), sizeof(dirname) - strlen(dirname), "/idmef-client.conf");
 
         prelude_client_profile_get_config_filename(profile, filename, sizeof(filename));
 
@@ -696,9 +719,9 @@ static int create_template_config_file(prelude_client_profile_t *profile)
         if ( ret == 0 )
                 return 0;
 
-        tfd = fopen(PRELUDE_CONFIG_DIR "/default/idmef-client.conf", "r");
+        tfd = fopen(dirname, "r");
         if ( ! tfd ) {
-                fprintf(stderr, "could not open '" PRELUDE_CONFIG_DIR "/default/idmef-client.conf' for reading: %s.\n", strerror(errno));
+                fprintf(stderr, "could not open '%s' for reading: %s.\n", dirname, strerror(errno));
                 return -1;
         }
 
@@ -1979,21 +2002,26 @@ static int list_cmd(int argc, char **argv)
         size_t size;
         struct stat st;
         struct dirent *dh;
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         struct group *gr;
         struct passwd *pw;
+#endif
         gnutls_datum data;
         prelude_string_t *str;
         unsigned int cert_max;
         int ret, i, permission;
         gnutls_x509_crt certs[1024];
-        char buf[1024], analyzerid[128], uidbuf[128], gidbuf[128];
+        char dirname[PATH_MAX];
+        char buf[1024], analyzerid[128], uidbuf[128] = { 0 }, gidbuf[128] = { 0 };
 
         setup_list_options();
         i = ret = prelude_option_read(parentopt, NULL, &argc, argv, &str, NULL);
 
-        dir = opendir(PRELUDE_CONFIG_DIR "/profile");
+        prelude_client_profile_get_profile_dirname(NULL, dirname, sizeof(dirname));
+
+        dir = opendir(dirname);
         if ( ! dir ) {
-                fprintf(stderr, "could not open '%s': %s.\n", PRELUDE_CONFIG_DIR "/profile", strerror(errno));
+                fprintf(stderr, "could not open '%s': %s.\n", dirname, strerror(errno));
                 return -1;
         }
 
@@ -2007,7 +2035,7 @@ static int list_cmd(int argc, char **argv)
                 if ( strcmp(dh->d_name, ".") == 0 || strcmp(dh->d_name, "..") == 0 )
                         continue;
 
-                snprintf(buf, sizeof(buf), PRELUDE_CONFIG_DIR "/profile/%s", dh->d_name);
+                snprintf(buf, sizeof(buf), "%s/%s", dirname, dh->d_name);
 
                 ret = stat(buf, &st);
                 if ( ret < 0 ) {
@@ -2016,6 +2044,7 @@ static int list_cmd(int argc, char **argv)
                         continue;
                 }
 
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
                 pw = getpwuid(st.st_uid);
                 if ( ! pw )
                         snprintf(uidbuf, sizeof(uidbuf), "%d", (int) st.st_uid);
@@ -2027,8 +2056,9 @@ static int list_cmd(int argc, char **argv)
                         snprintf(gidbuf, sizeof(gidbuf), "%d", (int) st.st_gid);
                 else
                         snprintf(gidbuf, sizeof(gidbuf), "%s", gr->gr_name);
+#endif
 
-                snprintf(buf, sizeof(buf), PRELUDE_CONFIG_DIR "/profile/%s/analyzerid", dh->d_name);
+                snprintf(buf, sizeof(buf), "%s/%s/analyzerid", dirname, dh->d_name);
                 fd = fopen(buf, "r");
                 if ( ! fd )
                         snprintf(analyzerid, sizeof(analyzerid), "n/a");
@@ -2042,7 +2072,7 @@ static int list_cmd(int argc, char **argv)
                         analyzerid[size - 1] = 0;
                 }
 
-                snprintf(buf, sizeof(buf), PRELUDE_CONFIG_DIR "/profile/%s/client.keycrt", dh->d_name);
+                snprintf(buf, sizeof(buf), "%s/%s/client.keycrt", dirname, dh->d_name);
                 ret = _prelude_load_file(buf, &data.data, &size);
                 if ( ret < 0 ) {
                         print_add(0, dh->d_name, uidbuf, gidbuf, analyzerid, "n/a", "n/a");
@@ -2169,7 +2199,7 @@ int main(int argc, char **argv)
         gnutls_global_init_extra();
 #endif
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         signal(SIGPIPE, SIG_IGN);
 #endif
 
