@@ -32,12 +32,27 @@
 #include <unistd.h>
 #include <sys/types.h>
 
+#include "prelude-config.h"
 #include "prelude-inttypes.h"
 
 #ifdef __cplusplus
  extern "C" {
 #endif
 
+
+#ifdef HAVE_UID_T
+typedef uid_t prelude_uid_t;
+#else
+typedef int prelude_uid_t;
+#endif
+
+#ifdef HAVE_GID_T
+typedef gid_t prelude_gid_t;
+#else
+typedef int prelude_gid_t;
+#endif
+
+
 typedef struct prelude_client_profile prelude_client_profile_t;
 
 int _prelude_client_profile_init(prelude_client_profile_t *cp);
@@ -50,6 +65,8 @@ void prelude_client_profile_destroy(prelude_client_profile_t *cp);
 
 void prelude_client_profile_get_config_filename(const prelude_client_profile_t *cp, char *buf, size_t size);
 
+void prelude_client_profile_get_default_config_dirname(const prelude_client_profile_t *cp, char *buf, size_t size);
+
 void prelude_client_profile_get_analyzerid_filename(const prelude_client_profile_t *cp, char *buf, size_t size);
 
 void prelude_client_profile_get_tls_key_filename(const prelude_client_profile_t *cp, char *buf, size_t size);
@@ -59,7 +76,7 @@ void prelude_client_profile_get_tls_server_ca_cert_filename(const prelude_client
 void prelude_client_profile_get_tls_server_keycert_filename(const prelude_client_profile_t *cp, char *buf, size_t size);
 
 void prelude_client_profile_get_tls_server_crl_filename(const prelude_client_profile_t *cp, char *buf, size_t size);
-         
+
 void prelude_client_profile_get_tls_client_keycert_filename(const prelude_client_profile_t *cp, char *buf, size_t size);
 
 void prelude_client_profile_get_tls_client_trusted_cert_filename(const prelude_client_profile_t *cp, char *buf, size_t size);
@@ -68,13 +85,13 @@ void prelude_client_profile_get_backup_dirname(const prelude_client_profile_t *c
 
 void prelude_client_profile_get_profile_dirname(const prelude_client_profile_t *cp, char *buf, size_t size);
 
-void prelude_client_profile_set_uid(prelude_client_profile_t *cp, uid_t uid);
+void prelude_client_profile_set_uid(prelude_client_profile_t *cp, prelude_uid_t uid);
 
-uid_t prelude_client_profile_get_uid(const prelude_client_profile_t *cp);
+prelude_uid_t prelude_client_profile_get_uid(const prelude_client_profile_t *cp);
 
-void prelude_client_profile_set_gid(prelude_client_profile_t *cp, uid_t gid);
+void prelude_client_profile_set_gid(prelude_client_profile_t *cp, prelude_uid_t gid);
 
-gid_t prelude_client_profile_get_gid(const prelude_client_profile_t *cp);
+prelude_gid_t prelude_client_profile_get_gid(const prelude_client_profile_t *cp);
 
 int prelude_client_profile_set_name(prelude_client_profile_t *cp, const char *name);
 
@@ -86,8 +103,12 @@ void prelude_client_profile_set_analyzerid(prelude_client_profile_t *cp, uint64_
 
 int prelude_client_profile_get_credentials(prelude_client_profile_t *cp, void **credentials);
 
+int prelude_client_profile_set_prefix(prelude_client_profile_t *cp, const char *prefix);
+
+void prelude_client_profile_get_prefix(const prelude_client_profile_t *cp, char *buf, size_t size);
+
 #ifdef __cplusplus
  }
 #endif
-         
+
 #endif
