@@ -31,7 +31,7 @@
  * This is mostly used by the libprelude detection routine in libprelude.m4
  * to make sure that the header version is consistant with the linked library.
  */
-#define LIBPRELUDE_VERSION "0.9.18.1"
+#define LIBPRELUDE_VERSION "0.9.21"
 
 #include "prelude-inttypes.h"
 #include "common.h"
@@ -61,6 +61,13 @@ const char *prelude_check_version(const char *req_version);
 
 int prelude_thread_init(void *future_use);
 
+void prelude_fork_prepare(void);
+
+void prelude_fork_parent(void);
+
+void prelude_fork_child(void);
+
+
 #ifdef __cplusplus
  }
 #endif
