@@ -35,19 +35,16 @@
 #include <assert.h>
 #include <errno.h>
 
-#ifndef WIN32
-# include <sys/utsname.h>
-#endif
-
 #include <gcrypt.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 
+#include "uname.h"
+#include "glthread/lock.h"
 
 #define PRELUDE_ERROR_SOURCE_DEFAULT PRELUDE_ERROR_SOURCE_CLIENT
 #include "prelude-error.h"
 
-#include "prelude-thread.h"
 #include "idmef.h"
 #include "common.h"
 #include "prelude-log.h"
@@ -118,7 +115,7 @@ struct prelude_client {
 
 
         prelude_msgbuf_t *msgbuf;
-        pthread_mutex_t msgbuf_lock;
+        gl_lock_t msgbuf_lock;
 
         prelude_ident_t *unique_ident;
 
@@ -280,7 +277,115 @@ static void setup_heartbeat_timer(prelude_client_t *client, int expire)
 }
 
 
-#ifndef WIN32
+static prelude_bool_t is_loopback_ipv6(struct in6_addr *addr)
+{
+        struct in6_addr lo;
+
+        inet_pton(AF_INET6, "::1", &lo);
+
+        return (memcmp(addr, &lo, sizeof(lo)) == 0) ? TRUE : FALSE;
+}
+
+
+static prelude_bool_t is_loopback_ipv4(struct in_addr *addr)
+{
+        return (ntohl(addr->s_addr) >> 24) == 127 ? TRUE : FALSE;
+}
+
+
+static prelude_bool_t is_loopback(int family, void *addr)
+{
+        if ( family == AF_INET )
+                return is_loopback_ipv4(addr);
+
+        else if ( family == AF_INET6 )
+                return is_loopback_ipv6(addr);
+
+        else
+                return FALSE;
+}
+
+
+static int set_analyzer_host_info(idmef_analyzer_t *analyzer, const char *node_str, const char *addr_str)
+{
+        int ret;
+        idmef_node_t *node;
+        idmef_address_t *addr;
+        prelude_string_t *str;
+
+        ret = idmef_analyzer_new_node(analyzer, &node);
+        if ( ret < 0 )
+                return ret;
+
+        ret = idmef_node_new_name(node, &str);
+        if ( ret < 0 )
+                return ret;
+
+        if ( prelude_string_is_empty(str) )
+                prelude_string_set_dup(str, node_str);
+
+        if ( ! (addr = idmef_node_get_next_address(node, NULL)) ) {
+                ret = idmef_node_new_address(node, &addr, 0);
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        ret = idmef_address_new_address(addr, &str);
+        if ( ret < 0 )
+                return ret;
+
+        if ( prelude_string_is_empty(str) )
+                prelude_string_set_dup(str, addr_str);
+
+        return 0;
+}
+
+
+static int get_fqdn(idmef_analyzer_t *analyzer, const char *nodename)
+{
+        int ret;
+        void *in_addr;
+        char addr[256], *addrp = NULL;
+        struct addrinfo hints, *ai, *ais;
+
+        prelude_log_debug(1, "Detected nodename: '%s'.\n", nodename);
+
+        memset(&hints, 0, sizeof(hints));
+        hints.ai_flags = AI_CANONNAME;
+
+        ret = getaddrinfo(nodename, NULL, &hints, &ai);
+        if ( ret < 0 )
+                return ret;
+
+        if ( ai->ai_canonname ) {
+                nodename = ai->ai_canonname;
+                prelude_log_debug(1, "Found canonical name: '%s'.\n", nodename);
+        }
+
+        for ( ais = ai; ai != NULL; ai = ai->ai_next ) {
+                in_addr = prelude_sockaddr_get_inaddr(ai->ai_addr);
+                if ( ! in_addr )
+                        continue;
+
+                if ( ! inet_ntop(ai->ai_family, in_addr, addr, sizeof(addr)) )
+                        continue;
+
+                if ( is_loopback(ai->ai_family, in_addr) )
+                        prelude_log_debug(1, "Ignoring loopback address: '%s'.\n", addr);
+                else {
+                        prelude_log_debug(1, "Found address: '%s'.\n", addr);
+                        addrp = addr;
+                        break;
+                }
+        }
+
+        ret = set_analyzer_host_info(analyzer, nodename, addrp);
+        freeaddrinfo(ais);
+
+        return ret;
+}
+
+
 static int get_sys_info(idmef_analyzer_t *analyzer)
 {
         int ret;
@@ -290,6 +395,8 @@ static int get_sys_info(idmef_analyzer_t *analyzer)
         if ( uname(&uts) < 0 )
                 return prelude_error_from_errno(errno);
 
+        get_fqdn(analyzer, uts.nodename);
+
         ret = prelude_string_new_dup(&str, uts.sysname);
         if ( ret < 0 )
                 return ret;
@@ -305,22 +412,6 @@ static int get_sys_info(idmef_analyzer_t *analyzer)
         return 0;
 }
 
-#else
-
-static int get_sys_info(idmef_analyzer_t *analyzer)
-{
-        int ret;
-        prelude_string_t *str;
-
-        ret = prelude_string_new_constant(&str, "Windows");
-        if ( ret < 0 )
-                return ret;
-
-        idmef_analyzer_set_ostype(analyzer, str);
-
-        return 0;
-}
-#endif
 
 static int fill_client_infos(prelude_client_t *client, const char *program)
 {
@@ -827,7 +918,7 @@ static int get_heartbeat_interval(prelude_option_t *opt, prelude_string_t *out,
 static int set_profile(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
 {
         int ret;
-        char buf[512];
+        char buf[PATH_MAX];
         prelude_client_t *client = context;
 
         ret = prelude_client_profile_set_name(client->profile, optarg);
@@ -880,6 +971,8 @@ static void _prelude_client_destroy(prelude_client_t *client)
 
 static int handle_client_error(prelude_client_t *client, int error)
 {
+        char *tmp = NULL;
+
         prelude_error_code_t code;
         prelude_error_source_t source;
 
@@ -887,8 +980,10 @@ static int handle_client_error(prelude_client_t *client, int error)
         source = prelude_error_get_source(error);
 
         if ( error < 0 && (code == PRELUDE_ERROR_PROFILE || source == PRELUDE_ERROR_SOURCE_CONFIG_ENGINE) ) {
-                char *tmp = strdup(_prelude_thread_get_error());
-                error = prelude_error_verbose(code, "%s\n%s", tmp, prelude_client_get_setup_error(client));
+                if ( _prelude_thread_get_error() )
+                        tmp = strdup(_prelude_thread_get_error());
+
+                error = prelude_error_verbose(PRELUDE_ERROR_PROFILE, "%s%s%s", tmp ? tmp : "", tmp ? "\n" : "", prelude_client_get_setup_error(client));
                 free(tmp);
         }
 
@@ -1048,7 +1143,7 @@ int prelude_client_new(prelude_client_t **client, const char *profile)
         if ( ! new )
                 return prelude_error_from_errno(errno);
 
-        prelude_thread_mutex_init(&new->msgbuf_lock, NULL);
+        gl_lock_init(new->msgbuf_lock);
         prelude_timer_init_list(&new->heartbeat_timer);
 
         new->flags = PRELUDE_CLIENT_FLAGS_HEARTBEAT|PRELUDE_CLIENT_FLAGS_CONNECT|PRELUDE_CLIENT_FLAGS_AUTOCONFIG;
@@ -1280,13 +1375,102 @@ void prelude_client_send_idmef(prelude_client_t *client, idmef_message_t *msg)
          * we need to hold a lock since asynchronous heartbeat
          * could write the message buffer at the same time we do.
          */
-        prelude_thread_mutex_lock(&client->msgbuf_lock);
+        gl_lock_lock(client->msgbuf_lock);
 
         _idmef_message_assign_missing(client, msg);
         idmef_message_write(msg, client->msgbuf);
         prelude_msgbuf_mark_end(client->msgbuf);
 
-        prelude_thread_mutex_unlock(&client->msgbuf_lock);
+        gl_lock_unlock(client->msgbuf_lock);
+}
+
+
+
+/**
+ * prelude_client_recv_msg:
+ * @client: Pointer to a #prelude_client_t object.
+ * @timeout: Number of millisecond to wait for a message.
+ * @msg: Pointer where the received #prelude_msg_t should be stored.
+ *
+ * Wait @timeout second for a message on @client connection pool.
+ *
+ * A @timeout of -1, mean prelude_client_recv_msg() will block until
+ * a message is received. A @timeout of 0 mean that it will return
+ * immediatly.
+ *
+ * Returns: 0 on timeout, a negative value on error, 1 on success.
+ */
+int prelude_client_recv_msg(prelude_client_t *client, int timeout, prelude_msg_t **msg)
+{
+        int ret;
+        prelude_msg_t *m = NULL;
+        prelude_connection_t *con;
+
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(msg, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        ret = prelude_connection_pool_recv(client->cpool, timeout, &con, &m);
+        if ( ret <= 0 )
+                return ret;
+
+        ret = prelude_client_handle_msg_default(client, m, client->msgbuf);
+        if ( ret == 0 ) {
+                prelude_msg_destroy(m);
+                return 0;
+        }
+
+        *msg = m;
+        return 1;
+}
+
+
+
+/**
+ * prelude_client_recv_idmef:
+ * @client: Pointer to a #prelude_client_t object.
+ * @timeout: Number of second to wait for a message.
+ * @idmef: Pointer where the received #idmef_message_t should be stored.
+ *
+ * Wait @timeout second for a message on @client connection pool.
+ *
+ * A @timeout of -1, mean prelude_client_recv_idmef() will block until
+ * a message is received. A @timeout of 0 mean that it will return
+ * immediatly.
+ *
+ * Returns: 0 on timeout, a negative value on error, 1 on success.
+ */
+int prelude_client_recv_idmef(prelude_client_t *client, int timeout, idmef_message_t **idmef)
+{
+        int ret;
+        prelude_msg_t *msg = NULL;
+
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(idmef, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! (client->permission & PRELUDE_CONNECTION_PERMISSION_IDMEF_READ) )
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC,
+                                             "Client should use 'idmef:r' permission to read IDMEF message");
+
+        ret = prelude_client_recv_msg(client, timeout, &msg);
+        if ( ret <= 0 )
+                return ret;
+
+        ret = idmef_message_new(idmef);
+        if ( ret < 0 ) {
+                prelude_msg_destroy(msg);
+                return ret;
+        }
+
+        ret = idmef_message_read(*idmef, msg);
+        if ( ret < 0 ) {
+                prelude_msg_destroy(msg);
+                idmef_message_destroy(*idmef);
+                return ret;
+        }
+
+        idmef_message_set_pmsg(*idmef, msg);
+
+        return 1;
 }
 
 
@@ -1648,12 +1832,12 @@ int prelude_client_handle_msg_default(prelude_client_t *client, prelude_msg_t *m
         /*
          * lock, handle the request, send reply.
          */
-        prelude_thread_mutex_lock(&client->msgbuf_lock);
+        gl_lock_lock(client->msgbuf_lock);
 
         ret = prelude_option_process_request(client, msg, msgbuf);
         prelude_msgbuf_mark_end(client->msgbuf);
 
-        prelude_thread_mutex_unlock(&client->msgbuf_lock);
+        gl_lock_unlock(client->msgbuf_lock);
 
         return ret;
 }
