@@ -34,6 +34,8 @@
 #include <errno.h>
 #include <assert.h>
 
+#include "glthread/lock.h"
+
 #include "common.h"
 #include "prelude-timer.h"
 #include "prelude-log.h"
@@ -44,6 +46,11 @@
 #include "prelude-option-wide.h"
 #include "prelude-failover.h"
 
+#if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
+# undef  FD_SETSIZE
+# define FD_SETSIZE 1024
+#endif
+
 #define PRELUDE_ERROR_SOURCE_DEFAULT PRELUDE_ERROR_SOURCE_CONNECTION_POOL
 #include "prelude-error.h"
 
@@ -101,7 +108,7 @@ typedef struct cnx {
 
 
 struct prelude_connection_pool {
-        pthread_mutex_t mutex;
+        gl_recursive_lock_t mutex;
 
         cnx_list_t *or_list;
         prelude_bool_t initialized;
@@ -275,10 +282,10 @@ again:
                         to.tv_sec = (timeout < 0) ? 1 : 0;
                 }
 
-                prelude_thread_mutex_lock(&pool->mutex);
+                gl_recursive_lock_lock(pool->mutex);
                 rfds = pool->fds;
                 nfd = pool->nfd;
-                prelude_thread_mutex_unlock(&pool->mutex);
+                gl_recursive_lock_unlock(pool->mutex);
 
                 ret = select(nfd, &rfds, NULL, NULL, &to);
                 if ( ret < 0 )
@@ -289,7 +296,7 @@ again:
         if ( ret == 0 )
                 return 0;
 
-        /* FIXME: prelude_thread_mutex_lock(&pool->mutex); */
+        gl_recursive_lock_lock(pool->mutex);
 
         for ( or = pool->or_list; or != NULL; or = or->or ) {
         for ( cnx = or->and; cnx != NULL; cnx = cnx->and ) {
@@ -311,7 +318,7 @@ again:
                         i--;
         }}
 
-        /* FIXME: prelude_thread_mutex_unlock(&pool->mutex); */
+        gl_recursive_lock_unlock(pool->mutex);
         global_event_handler(pool, global_event);
 
         if ( pool->connection_string_changed )
@@ -390,6 +397,18 @@ static void init_cnx_timer(cnx_t *cnx)
 }
 
 
+static void destroy_connection_single(cnx_t *cnx)
+{
+        prelude_timer_destroy(&cnx->timer);
+        prelude_connection_destroy(cnx->cnx);
+
+        if ( cnx->failover )
+                prelude_failover_destroy(cnx->failover);
+
+        free(cnx);
+}
+
+
 static void connection_list_destroy(cnx_list_t *clist)
 {
         void *bkp;
@@ -399,14 +418,7 @@ static void connection_list_destroy(cnx_list_t *clist)
 
                 for ( cnx = clist->and; cnx != NULL; cnx = bkp ) {
                         bkp = cnx->and;
-
-                        prelude_timer_destroy(&cnx->timer);
-                        prelude_connection_destroy(cnx->cnx);
-
-                        if ( cnx->failover )
-                                prelude_failover_destroy(cnx->failover);
-
-                        free(cnx);
+                        destroy_connection_single(cnx);
                 }
 
                 bkp = clist->or;
@@ -623,7 +635,7 @@ static void connection_timer_expire(void *data)
         cnx_t *cnx = data;
         prelude_connection_pool_t *pool = cnx->parent->parent;
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
 
         ret = prelude_connection_connect(cnx->cnx, pool->client_profile, pool->permission);
         if ( ret >= 0 ) {
@@ -644,7 +656,7 @@ static void connection_timer_expire(void *data)
         prelude_timer_reset(&cnx->timer);
 
 out:
-        prelude_thread_mutex_unlock(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
 }
 
 
@@ -652,22 +664,22 @@ static int new_connection(cnx_t **ncnx, prelude_client_profile_t *cp, cnx_list_t
                           prelude_connection_t *cnx, prelude_connection_pool_flags_t flags)
 {
         int ret;
-        cnx_t *new;
+        cnx_t *nc;
         char *dirname, buf[PATH_MAX];
 
-        *ncnx = new = malloc(sizeof(*new));
-        if ( ! new )
+        nc = malloc(sizeof(*nc));
+        if ( ! nc )
                 return prelude_error_from_errno(errno);
 
-        new->msg = NULL;
-        new->failover = NULL;
-        new->parent = clist;
-        prelude_timer_init_list(&new->timer);
+        nc->msg = NULL;
+        nc->failover = NULL;
+        nc->parent = clist;
+        prelude_timer_init_list(&nc->timer);
 
         if ( flags & PRELUDE_CONNECTION_POOL_FLAGS_RECONNECT ) {
-                prelude_timer_set_data(&new->timer, new);
-                prelude_timer_set_expire(&new->timer, INITIAL_EXPIRATION_TIME);
-                prelude_timer_set_callback(&new->timer, connection_timer_expire);
+                prelude_timer_set_data(&nc->timer, nc);
+                prelude_timer_set_expire(&nc->timer, INITIAL_EXPIRATION_TIME);
+                prelude_timer_set_callback(&nc->timer, connection_timer_expire);
         }
 
         if ( flags & PRELUDE_CONNECTION_POOL_FLAGS_FAILOVER ) {
@@ -675,22 +687,25 @@ static int new_connection(cnx_t **ncnx, prelude_client_profile_t *cp, cnx_list_t
 
                 ret = get_connection_backup_path(cnx, buf, &dirname);
                 if ( ret < 0 )
-                        return ret;
+                        goto err;
 
-                ret = prelude_failover_new(&new->failover, dirname);
+                ret = prelude_failover_new(&nc->failover, dirname);
                 free(dirname);
-                if ( ret < 0 ) {
-                        free(new);
-                        return ret;
-                }
+                if ( ret < 0 )
+                        goto err;
         }
 
-        new->cnx = cnx;
-        new->and = NULL;
+        nc->cnx = cnx;
+        nc->and = NULL;
         clist->total++;
         prelude_linked_object_add(&clist->parent->all_cnx, (prelude_linked_object_t *) cnx);
+        *ncnx = nc;
 
         return 0;
+
+    err:
+        free(nc);
+        return ret;
 }
 
 
@@ -874,9 +889,9 @@ void prelude_connection_pool_broadcast(prelude_connection_pool_t *pool, prelude_
         prelude_return_if_fail(pool);
         prelude_return_if_fail(msg);
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
         walk_manager_lists(pool, msg);
-        prelude_thread_mutex_unlock(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
 }
 
 
@@ -894,9 +909,9 @@ void prelude_connection_pool_broadcast_async(prelude_connection_pool_t *pool, pr
         prelude_return_if_fail(pool);
         prelude_return_if_fail(msg);
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
         pool->refcount++;
-        prelude_thread_mutex_unlock(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
 
         prelude_async_set_callback((prelude_async_object_t *) msg, &broadcast_async_cb);
         prelude_async_set_data((prelude_async_object_t *) msg, pool);
@@ -923,7 +938,7 @@ int prelude_connection_pool_init(prelude_connection_pool_t *pool)
         int ret = 0, event = 0;
         char dirname[PATH_MAX], buf[PATH_MAX];
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
 
         prelude_return_val_if_fail(pool, prelude_error(PRELUDE_ERROR_ASSERTION));
 
@@ -994,7 +1009,7 @@ int prelude_connection_pool_init(prelude_connection_pool_t *pool)
         pool->initialized = TRUE;
 
 err:
-        prelude_thread_mutex_unlock(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
         return ret;
 }
 
@@ -1030,7 +1045,7 @@ int prelude_connection_pool_new(prelude_connection_pool_t **ret,
 
         prelude_list_init(&new->all_cnx);
         prelude_timer_init_list(&new->timer);
-        prelude_thread_mutex_init(&new->mutex, NULL);
+        gl_recursive_lock_init(new->mutex);
 
         return 0;
 }
@@ -1046,10 +1061,10 @@ void prelude_connection_pool_destroy(prelude_connection_pool_t *pool)
 {
         prelude_return_if_fail(pool);
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
 
         if ( --pool->refcount != 0 ) {
-                prelude_thread_mutex_unlock(&pool->mutex);
+                gl_recursive_lock_unlock(pool->mutex);
                 return;
         }
 
@@ -1063,8 +1078,8 @@ void prelude_connection_pool_destroy(prelude_connection_pool_t *pool)
         if ( pool->failover )
                 prelude_failover_destroy(pool->failover);
 
-        prelude_thread_mutex_unlock(&pool->mutex);
-        prelude_thread_mutex_destroy(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
+        gl_recursive_lock_destroy(pool->mutex);
 
         free(pool);
 }
@@ -1112,7 +1127,7 @@ int prelude_connection_pool_add_connection(prelude_connection_pool_t *pool, prel
         prelude_return_val_if_fail(pool, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(cnx, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
 
         if ( ! pool->or_list ) {
                 ret = create_connection_list(&pool->or_list, pool);
@@ -1142,7 +1157,41 @@ int prelude_connection_pool_add_connection(prelude_connection_pool_t *pool, prel
         }
 
 out:
-        prelude_thread_mutex_unlock(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
+        return ret;
+}
+
+
+
+/**
+ * prelude_connection_pool_del_connection:
+ * @pool: Pointer to a #prelude_connection_pool_t object.
+ * @cnx: Pointer to a #prelude_connection_t object to remove from @pool.
+ *
+ * Remove @cnx from @pool of connections.
+ *
+ * Returns: 0 on success, a negative value if an error occured.
+ */
+int prelude_connection_pool_del_connection(prelude_connection_pool_t *pool, prelude_connection_t *cnx)
+{
+        cnx_t *c;
+        int ret = 0;
+
+        prelude_return_val_if_fail(pool, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(cnx, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        gl_recursive_lock_lock(pool->mutex);
+
+        c = search_cnx(pool, cnx);
+        if ( ! c ) {
+                ret = prelude_error_verbose(PRELUDE_ERROR_GENERIC, "Connection is not within pool");
+                goto out;
+        }
+
+        destroy_connection_single(c);
+
+out:
+        gl_recursive_lock_unlock(pool->mutex);
         return ret;
 }
 
@@ -1230,7 +1279,7 @@ int prelude_connection_pool_set_connection_dead(prelude_connection_pool_t *pool,
         prelude_return_val_if_fail(pool, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(cnx, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
 
         c = search_cnx(pool, cnx);
         if ( ! c ) {
@@ -1245,7 +1294,7 @@ int prelude_connection_pool_set_connection_dead(prelude_connection_pool_t *pool,
         set_state_dead(c, 0, FALSE, FALSE);
 
 out:
-        prelude_thread_mutex_unlock(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
         return ret;
 }
 
@@ -1274,7 +1323,7 @@ int prelude_connection_pool_set_connection_alive(prelude_connection_pool_t *pool
         prelude_return_val_if_fail(pool, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(cnx, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
 
         c = search_cnx(pool, cnx);
         if ( ! c ) {
@@ -1288,7 +1337,7 @@ int prelude_connection_pool_set_connection_alive(prelude_connection_pool_t *pool
         ret = set_state_alive(c, FALSE);
 
 out:
-        prelude_thread_mutex_unlock(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
         return ret;
 }
 
@@ -1322,7 +1371,7 @@ int prelude_connection_pool_set_connection_string(prelude_connection_pool_t *poo
         if ( ! new )
                 return prelude_error_from_errno(errno);
 
-        prelude_thread_mutex_lock(&pool->mutex);
+        gl_recursive_lock_lock(pool->mutex);
 
         if ( pool->connection_string )
                 free(pool->connection_string);
@@ -1330,7 +1379,7 @@ int prelude_connection_pool_set_connection_string(prelude_connection_pool_t *poo
         pool->connection_string = new;
         pool->connection_string_changed = TRUE;
 
-        prelude_thread_mutex_unlock(&pool->mutex);
+        gl_recursive_lock_unlock(pool->mutex);
 
         return 0;
 }
