@@ -94,6 +94,8 @@ int _prelude_connection_keepalive_intvl = 0;
 struct prelude_connection {
         PRELUDE_LINKED_OBJECT;
 
+        int refcount;
+
         char *saddr;
         unsigned int sport;
 
@@ -108,6 +110,7 @@ struct prelude_connection {
         prelude_connection_permission_t permission;
 
         void *data;
+        prelude_msg_t *msg;
 
         prelude_connection_state_t state;
 };
@@ -200,7 +203,6 @@ static int is_tcp_connection_still_established(prelude_io_t *pio)
 
 
 
-#ifndef WIN32
 static void set_single_socket_option(int sock, const char *name, int level, int option, int value)
 {
         int ret;
@@ -241,8 +243,6 @@ static void set_inet_socket_option(int sock, struct sockaddr *sa)
 # endif
 }
 
-#endif
-
 
 
 /*
@@ -257,7 +257,7 @@ static int generic_connect(struct sockaddr *sa, socklen_t salen)
         if ( sock < 0 )
                 return prelude_error_from_errno(errno);
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         fcntl(sock, F_SETFD, fcntl(sock, F_GETFD) | FD_CLOEXEC);
 
         ret = fcntl(sock, F_SETOWN, getpid());
@@ -265,11 +265,11 @@ static int generic_connect(struct sockaddr *sa, socklen_t salen)
                 close(sock);
                 return prelude_error_from_errno(errno);
         }
+#endif
 
         set_single_socket_option(sock, "SO_KEEPALIVE", SOL_SOCKET, SO_KEEPALIVE, 1);
         if ( sa->sa_family != AF_UNIX )
                 set_inet_socket_option(sock, sa);
-#endif
 
         ret = connect(sock, sa, salen);
         if ( ret < 0 ) {
@@ -381,7 +381,7 @@ static int start_inet_connection(prelude_connection_t *cnx,
 
 
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
 static int start_unix_connection(prelude_connection_t *cnx,
                                  prelude_connection_permission_t reqperms, prelude_client_profile_t *profile)
 {
@@ -415,7 +415,7 @@ static int do_connect(prelude_connection_t *cnx,
                 ret = start_inet_connection(cnx, reqperms, profile);
         }
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         else {
                 prelude_log(PRELUDE_LOG_INFO, "Connecting to %s (UNIX) prelude Manager server.\n",
                             ((struct sockaddr_un *) cnx->sa)->sun_path);
@@ -540,12 +540,12 @@ static int resolve_addr(prelude_connection_t *cnx, const char *addr)
 {
         struct addrinfo *ai;
         int ret, ai_family, ai_addrlen;
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         struct sockaddr_un *un;
 #endif
 
         if ( is_unix_addr(cnx, addr) ) {
-#ifdef WIN32
+#if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "UNIX socket are not supported under this environment");
 #else
                 ai_family = AF_UNIX;
@@ -576,7 +576,7 @@ static int resolve_addr(prelude_connection_t *cnx, const char *addr)
                 freeaddrinfo(ai);
         }
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         else {
                 un = (struct sockaddr_un *) cnx->sa;
                 strncpy(un->sun_path, cnx->daddr, sizeof(un->sun_path));
@@ -602,6 +602,9 @@ void prelude_connection_destroy(prelude_connection_t *conn)
 {
         prelude_return_if_fail(conn);
 
+        if ( --conn->refcount > 0 )
+                return;
+
         destroy_connection_fd(conn);
 
         free(conn->daddr);
@@ -619,7 +622,7 @@ int prelude_connection_new(prelude_connection_t **out, const char *addr)
 
         prelude_return_val_if_fail(addr, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-#ifndef WIN32
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         signal(SIGPIPE, SIG_IGN);
 #endif
 
@@ -627,6 +630,8 @@ int prelude_connection_new(prelude_connection_t **out, const char *addr)
         if ( ! new )
                 return prelude_error_from_errno(errno);
 
+        new->refcount = 1;
+
         ret = prelude_io_new(&new->fd);
         if ( ret < 0 ) {
                 free(new);
@@ -774,6 +779,43 @@ int prelude_connection_recv(prelude_connection_t *cnx, prelude_msg_t **msg)
 }
 
 
+int prelude_connection_recv_idmef(prelude_connection_t *con, idmef_message_t **idmef)
+{
+        int ret;
+
+        ret = prelude_connection_recv(con, &con->msg);
+        if ( ret < 0 ) {
+                if ( prelude_error_get_code(ret) != PRELUDE_ERROR_EAGAIN )
+                        con->msg = NULL;
+
+                return ret;
+        }
+        if ( prelude_msg_get_tag(con->msg) != PRELUDE_MSG_IDMEF ) {
+                prelude_msg_destroy(con->msg);
+                con->msg = NULL;
+                return prelude_error_from_errno(EINVAL);
+        }
+
+        ret = idmef_message_new(idmef);
+        if ( ret < 0 ) {
+                prelude_msg_destroy(con->msg);
+                con->msg = NULL;
+                return ret;
+        }
+
+        ret = idmef_message_read(*idmef, con->msg);
+        if ( ret < 0 ) {
+                idmef_message_destroy(*idmef);
+                prelude_msg_destroy(con->msg);
+                con->msg = NULL;
+                return ret;
+        }
+
+        idmef_message_set_pmsg(*idmef, con->msg);
+        con->msg = NULL;
+
+        return ret;
+}
 
 
 /**
@@ -1054,3 +1096,21 @@ prelude_connection_permission_t prelude_connection_get_permission(prelude_connec
 }
 
 
+/**
+ * prelude_connection_ref:
+ * @conn: Pointer to a #prelude_connection_t object to reference.
+ *
+ * Increases @conn reference count.
+ *
+ * prelude_connection_destroy() will decrease the refcount until it
+ * reaches 0, at which point @conn will be destroyed.
+ *
+ * Returns: @conn.
+ */
+prelude_connection_t *prelude_connection_ref(prelude_connection_t *conn)
+{
+        prelude_return_val_if_fail(conn, NULL);
+
+        conn->refcount++;
+        return conn;
+}
