@@ -33,13 +33,15 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <fcntl.h>
+#include <signal.h>
 #include <errno.h>
 
 #include <assert.h>
 #include <gcrypt.h>
 
-#include "common.h"
+#include "glthread/thread.h"
 
+#include "common.h"
 #include "prelude-log.h"
 #include "prelude-io.h"
 #include "prelude-msg.h"
@@ -48,10 +50,6 @@
 #define PRELUDE_ERROR_SOURCE_DEFAULT PRELUDE_ERROR_SOURCE_FAILOVER
 #include "prelude-error.h"
 
-#ifdef WIN32
-# define mkdir(x, y) mkdir(x)
-#endif
-
 
 #define FAILOVER_CHECKSUM_SIZE       4
 #define FAILOVER_JOURNAL_ENTRY_SIZE 20
@@ -84,13 +82,13 @@ static void mask_signal(sigset_t *oldmask)
         sigset_t newmask;
 
         prelude_return_if_fail( sigfillset(&newmask) == 0 );
-        prelude_return_if_fail( pthread_sigmask(SIG_BLOCK, &newmask, oldmask) == 0 );
+        prelude_return_if_fail( glthread_sigmask(SIG_BLOCK, &newmask, oldmask) == 0 );
 }
 
 
 static void unmask_signal(sigset_t *oldmask)
 {
-        prelude_return_if_fail( pthread_sigmask(SIG_SETMASK, oldmask, NULL) == 0 );
+        prelude_return_if_fail( glthread_sigmask(SIG_SETMASK, oldmask, NULL) == 0 );
 }
 
 
@@ -216,7 +214,7 @@ static int journal_check(prelude_failover_t *failover, failover_journal_entry_t
                 return -1;
         }
 
-        prelude_log_debug(7, "rindex=%" PRELUDE_PRIu64 " size=%" PRELUDE_PRIu64 "\n", jentry->value.rindex, wst->st_size);
+        prelude_log_debug(7, "rindex=%" PRELUDE_PRIu64 " size=%" PRELUDE_PRId64 "\n", jentry->value.rindex, (int64_t) wst->st_size);
         if ( jentry->value.rindex > wst->st_size ) {
                 /*
                  * Latest journal entry has a read index that is higher than the size
@@ -269,7 +267,7 @@ static int journal_read_last_entry(prelude_failover_t *failover,
                 if ( ret < 0 )
                         return ret;
 
-                prelude_log_debug(7, "[%" PRELUDE_PRIu64 "] found jentry with count=%" PRELUDE_PRIu64 " rindex=%" PRELUDE_PRIu64 "\n", offset, jentry->value.count, jentry->value.rindex);
+                prelude_log_debug(7, "[%" PRELUDE_PRId64 "] found jentry with count=%" PRELUDE_PRIu64 " rindex=%" PRELUDE_PRIu64 "\n", (int64_t) offset, jentry->value.count, jentry->value.rindex);
 
                 ret = journal_check(failover, jentry, wst);
 
@@ -296,6 +294,10 @@ static int journal_initialize(prelude_failover_t *failover, const char *filename
         if ( failover->jfd < 0 )
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not open '%s': %s", filename, strerror(errno));
 
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
+        fcntl(failover->jfd, F_SETFD, fcntl(failover->jfd, F_GETFD) | FD_CLOEXEC);
+#endif
+
         ret = fstat(failover->jfd, &jst);
         if ( ret < 0 )
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not stat failover journal: %s", strerror(errno));
@@ -330,6 +332,7 @@ static int journal_initialize(prelude_failover_t *failover, const char *filename
 }
 
 
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
 static int lock_cmd(const char *filename, int fd, int cmd, int type)
 {
         int ret;
@@ -351,20 +354,23 @@ static int lock_cmd(const char *filename, int fd, int cmd, int type)
 
         return 1;
 }
+#endif
 
 
 
 static int open_exclusive(const char *filename, int flags, int *fd)
 {
-        int ret;
+        int ret = 1;
 
         *fd = open(filename, flags, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
         if ( *fd < 0 )
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "error opening '%s': %s", filename, strerror(errno));
 
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
         ret = lock_cmd(filename, *fd, F_SETLK, F_RDLCK|F_WRLCK);
         if ( ret <= 0 )
                 close(*fd);
+#endif
 
         return ret;
 }
@@ -519,6 +525,10 @@ int prelude_failover_new(prelude_failover_t **out, const char *dirname)
                 return wfd;
         }
 
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
+        fcntl(wfd, F_SETFD, fcntl(wfd, F_GETFD) | FD_CLOEXEC);
+#endif
+
         rfd = open(filename, O_RDONLY);
         if ( rfd < 0 ) {
                 umask(mode);
@@ -526,6 +536,10 @@ int prelude_failover_new(prelude_failover_t **out, const char *dirname)
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not open '%s' for reading: %s", filename, strerror(errno));
         }
 
+#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
+        fcntl(rfd, F_SETFD, fcntl(rfd, F_GETFD) | FD_CLOEXEC);
+#endif
+
         new = calloc(1, sizeof(*new));
         if ( ! new ) {
                 umask(mode);
